%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Conversion of Maple file ``xple9-7-1.mws'' into Matlab file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

syms th1 th2 th3 th4 th5 th6 real
pi = sym('pi');

% The DH parameters of the Fanuc Arc Mate, as per Table 5.2, are reproduced below. 
% Vectors a[i] in FRMS are labelled here q[i], to avoid confusion with
% scalars a[i].

% DATA

hfigM = findobj('Tag','INPUT');
data = getappdata(hfigM, 'DATA');

%%%%%%%% FANUC
%a = [200 600 130 0 0 0]; 
%b = [810 0 -30 550 100 100];
%alpha = [pi/2 0 pi/2 pi/2 pi/2 0];
% theta = [th1 th2 th3 th4 th5 th6];

%%%%%%%% LI Manipulator
% a = [120 1760 70 880 390 930]; 
% b = [0 890 250 -430 500 -1340];
% alpha = [-57*pi/180 35*pi/180 95*pi/180 79*pi/180 -75*pi/180 -pi/2];
% theta = [th1 th2 th3 th4 th5 th6];

%%%%%%%% The general case
a = [data.a1 data.a2 data.a3 data.a4 data.a5 data.a6];
b = [data.b1 data.b2 data.b3 data.b4 data.b5 data.b6];
alpha = [data.alpha1 data.alpha2 data.alpha3 data.alpha4 data.alpha5 data.alpha6]*pi/180;
theta = [th1 th2 th3 th4 th5 th6];


% The CHARACTERISTIC LENGTH found in MD-05-1171 paper
L = data.CharacL;

setappdata(findobj('Tag','INPUT'), 'DATA', data);

a = a/L;
b = b/L;

% eq(9.2)
for i = 1 : 6
    q(i,:) = [a(i)*cos(theta(i)) a(i)*sin(theta(i)) b(i)];
    Q(1,:,i) = [cos(theta(i)) -cos(alpha(i))*sin(theta(i)) sin(alpha(i))*sin(theta(i))];
    Q(2,:,i) = [sin(theta(i)) cos(alpha(i))*cos(theta(i)) -sin(alpha(i))*cos(theta(i))];
    Q(3,:,i) = [0 sin(alpha(i)) cos(alpha(i))];
end

% Factor matrix Q_i as  Q=Z_iX_i, where X_i and Z_i are two reflections,
% as per eq. (4.2b):  
for i = 1 : 6
    X(:,:,i) = [1 0 0; 0 -cos(alpha(i)) sin(alpha(i)); 0 sin(alpha(i)), cos(alpha(i))];
    Z(:,:,i) = [cos(theta(i)) sin(theta(i)) 0 ; sin(theta(i)) -cos(theta(i)) 0 ; 0 0 1];
end

% Vectors c[i], of eq. (4.3e), and g[i], of eq.(9.9), are given below.
% The notation c[i] is used instead of b[i]  in order to avoid confusion with scalar b[i]. 
% No confusion with cos(theta[i]) =c[i] in FRMS, because we don't use  this shorthand notation here.
for i = 1 : 6
    c(:,i) = [a(i) ; b(i)*sin(alpha(i)) ; b(i)*cos(alpha(i))];
    g(:,i) = [a(i) ; 0 ; b(i)];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55555
% DATA

hfigM = findobj('Tag','INPUT');
data = getappdata(hfigM, 'DATA');

% The position of the EE operation point P, of position vector p, and the EE orientation given by the rotation matrix Rot: 
x = data.x/L;
y = data.y/L;
z = data.z/L;
p = [x; y; z];
Rot = [data.Rot11 data.Rot12 data.Rot13 ; data.Rot21 data.Rot22 data.Rot23 ; data.Rot31 data.Rot32 data.Rot33];

setappdata(findobj('Tag','INPUT'), 'DATA', data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Derivation of the Fundamental Closure Equations 
rho = p - Rot*c(:,6);

% o6        - eq. (9.6)  
% sigma  - eq.(9.18a)   
% u5        - eq. (9.7)     
%  f        - eq. (9.19c)   
% gg      - eq. (9.19d)    
% h         - eq. (9.19a)      
% i        - eq. (9.19b)    
% (both h and i are unit vectors):
o6 = Q(3,:,6);
sigma = Rot*o6';  % sigma is sigma_[6] in FRMS

u5 = Q(:,3,5);
h = Q(:,:,3)*Q(:,:,4)*u5;
i = Q(:,:,2)'*Q(:,:,1)'*sigma;
f = Q(:,:,3)*(c(:,3) + Q(:,:,4)*c(:,4) + Q(:,:,4)*Q(:,:,5)*c(:,5));
gg = Q(:,:,2)'*(Q(:,:,1)'*rho - c(:,1)) - c(:,2);      % gg is g in FRMS

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The Univariate-Polynomial Approach

% f-tilde (ft)--- eq.(9.26a)  
% h-tilde (ht)---eq.(9.26b)
% r-tilde (rt)---eq.(9.26c)   
% n-tilde (nt)---eq.(9.26d) 
ft = Z(:,:,3)*f;
ht = Q(:,:,1)'*rho - c(:,1);
rt = X(:,:,3)*Q(:,:,4)*u5;
nt = Q(:,:,1)'*sigma;

% Derive Raghavan and Roth's eqs. (9.31a--9.31 f): 
% f_bar (fb)---eq. (9.30a)   
% g_bar (gb)--- eq. (9.30b)
% h_bar (hb)--- eq. (9.30c) 
% i_bar (ib)--- eq. (9.30d)  
fb = X(:,:,2)*(Z(:,:,3)*ft + c(:,2));
gb = Z(:,:,2)*ht;
hb = X(:,:,2)*Z(:,:,3)*rt;
ib = Z(:,:,2)*nt;

% LHS and RHS of eq.(9.31c)   
dfb = maple('simplify',fb'*fb);
dgb = maple('simplify',gb'*gb);

% LHS and RHS of eq.(9.31d)  
dfhb = maple('simplify',fb'*hb);
dgib = maple('simplify',gb'*ib);

% LHS and RHS of eq.(9.31e)  
cfhb = maple('simplify',cross(fb,hb));
cgib = maple('simplify',cross(gb,ib));

% LHS and RHS of eq.(9.31f)   
reffhb = maple('simplify',dfb'*hb - 2*dfhb'*fb);
% the reflection of the h_bar wrs to the plane 
% with the unit normal f_bar
refgib = maple('simplify',dgb'*ib - 2*dgib'*gb);
% the reflection of i_bar wrs to the plane with 
% the unit normal g_bar 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Derive the contour equations by eliminating three of the remaining
% five unknown joint angles from the closure equations.
% 
% Start by deriving matrices P_bar and R_bar, eq.(9.32), 
% using eqs.(9.31a)--(9.31f) 
% 
% Define vectors  x12 and x45, eqs.(9.25)  as  8- and 
% 9-dimensional vectors, correspondingly

x12 = [sin(theta(1))*sin(theta(2)) sin(theta(1))*cos(theta(2)) cos(theta(1))*sin(theta(2)) cos(theta(1))*cos(theta(2)) sin(theta(1)) cos(theta(1)) sin(theta(2)) cos(theta(2))]';
x45 = [sin(theta(4))*sin(theta(5)) sin(theta(4))*cos(theta(5)) cos(theta(4))*sin(theta(5)) cos(theta(4))*cos(theta(5)) sin(theta(4)) cos(theta(4)) sin(theta(5)) cos(theta(5)) 1]';

% and vector x12ex which is a 9-dimensional vectors defined as vector x12 with added 9th 
% component equal to 1
x12ex = [sin(theta(1))*sin(theta(2)) sin(theta(1))*cos(theta(2)) cos(theta(1))*sin(theta(2)) cos(theta(1))*cos(theta(2)) sin(theta(1)) cos(theta(1)) sin(theta(2)) cos(theta(2)) 1]';

% and obtain vectors
RBx12ex = [gb(1) gb(2) gb(3) ib(1) ib(2) ib(3) dgb dgib cgib(1) cgib(2) cgib(3) refgib(1) refgib(2) refgib(3)]';
RBx12ex = maple('simplify',RBx12ex);


PBx45_pr = [fb(1) fb(2) fb(3) hb(1) hb(2) hb(3) dfb dfhb cfhb(1) cfhb(2) cfhb(3) reffhb(1) reffhb(2) reffhb(3)]'; 
PBx45_pr = maple('simplify',PBx45_pr);

% Then, obtain matrices RBex and PB_pr as the coefficients of vectors x12ex and x45
% in RBx12ex and PBx45_pr, respectively
for i = 1 : 14
    RBex(i,1) = maple('coeff',maple('coeff',RBx12ex(i),sin(theta(1))),sin(theta(2)));
    RBex(i,2) = maple('coeff',maple('coeff',RBx12ex(i),sin(theta(1))),cos(theta(2)));
    RBex(i,3) = maple('coeff',maple('coeff',RBx12ex(i),cos(theta(1))),sin(theta(2)));
    RBex(i,4) = maple('coeff',maple('coeff',RBx12ex(i),cos(theta(1))),cos(theta(2)));
    RBex(i,5) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',RBx12ex(i),sin(theta(1))),cos(theta(1)),0),sin(theta(2)),0),cos(theta(2)),0);
    RBex(i,6) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',RBx12ex(i),cos(theta(1))),sin(theta(1)),0),sin(theta(2)),0),cos(theta(2)),0);
    RBex(i,7) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',RBx12ex(i),sin(theta(2))),cos(theta(1)),0),sin(theta(1)),0),cos(theta(2)),0);
    RBex(i,8) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',RBx12ex(i),cos(theta(2))),sin(theta(2)),0),sin(theta(1)),0),cos(theta(1)),0);
    RBex(i,9) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',RBx12ex(i),sin(theta(1)),0),cos(theta(1)),0),sin(theta(2)),0),cos(theta(2)),0); 
end

RBex = maple('simplify',RBex);

for i = 1 : 14
    PB_pr(i,1) = maple('coeff',maple('coeff',PBx45_pr(i),sin(theta(4))),sin(theta(5)));
    PB_pr(i,2) = maple('coeff',maple('coeff',PBx45_pr(i),sin(theta(4))),cos(theta(5)));
    PB_pr(i,3) = maple('coeff',maple('coeff',PBx45_pr(i),cos(theta(4))),sin(theta(5)));
    PB_pr(i,4) = maple('coeff',maple('coeff',PBx45_pr(i),cos(theta(4))),cos(theta(5)));
    PB_pr(i,5) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',PBx45_pr(i),sin(theta(4))),cos(theta(4)),0),sin(theta(5)),0),cos(theta(5)),0);
    PB_pr(i,6) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',PBx45_pr(i),cos(theta(4))),sin(theta(4)),0),sin(theta(5)),0),cos(theta(5)),0);
    PB_pr(i,7) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',PBx45_pr(i),sin(theta(5))),cos(theta(4)),0),sin(theta(4)),0),cos(theta(5)),0);
    PB_pr(i,8) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',PBx45_pr(i),cos(theta(5))),sin(theta(5)),0),sin(theta(4)),0),cos(theta(4)),0);
    PB_pr(i,9) = maple('coeff',maple('coeff',maple('coeff',maple('coeff',PBx45_pr(i),sin(theta(4)),0),cos(theta(4)),0),sin(theta(5)),0),cos(theta(5)),0); 
end

PB_pr = maple('simplify',PB_pr);


% % Verify derivation of matrices RBex and PB_pr
checkRBex = maple('simplify',RBex*x12ex - RBx12ex);
checkPB_pr = maple('simplify',PB_pr*x45 - PBx45_pr);

checkRBex_test = vpa(checkRBex,4)
checkPB_pr_test = vpa(checkPB_pr,4)

% Now, derive the 14x8 matrix RB from matrix of RBx12ex with last column
% deleted:
RB = RBex(1:14,1:8);     % RB is R_bar in the RHS of eq.(9.32)
nine = RBex(:,9);        % ninth column of RBex

% PB is the 14x9 submatrix of matrix of PB_pr, with its last column being
% the last column of matrix PB_pr minus the last column of matrix of RBex (vector nine) 
PB = PB_pr;
PB(:,9) = PB(:,9)-nine;

% and vector x12t, x12-tilde, of eq.(9.35):
x12t = [sin(theta(1))*sin(theta(2)) sin(theta(1))*cos(theta(2)) cos(theta(1))*sin(theta(2)) cos(theta(1))*cos(theta(2)) sin(theta(2)) cos(theta(2))]';

% Partition eq.(9.32) into two groups by deriving:
C1 = [RB(3,:);RB(6,:);RB(7,:);RB(8,:);RB(11,:);RB(14,:)];
C2 = C1(:,5:8);
C = C2(:,1:2);

PuB = [PB(3,:);PB(6,:);PB(7,:);PB(8,:);PB(11,:);PB(14,:)];

% where C is a 6x2 constant matrix that contains the nonzero entries
% in rows 3,6,7,8,11, and 14 of matrix R_bar, while PuB is the matrix on LHS of eq. (9.34a) 
% 
% Then, derive an 8x6 matrix A from RB, and compute PlB as the matrix on LHS of eq. (9.34b) 

AA = [RB(1,:);RB(2,:);RB(4,:);RB(5,:);RB(9,:);RB(10,:);RB(12,:);RB(13,:)];
A = [AA(:,1:4) AA(:,7:8)];

PlB = [PB(1,:);PB(2,:);PB(4,:);PB(5,:);PB(9,:);PB(10,:);PB(12,:);PB(13,:)];


detCueval = 0;
k = 1;
m = 1;

while abs(detCueval) < 0.00001
    
    if k == 5 & m == 6
        break
        
    elseif m < 6
        m = m + 1;
    
        Cu(1,1) = C(k,1); 
        Cu(1,2) = C(k,2); 
        Cu(2,1) = C(m,1); 
        Cu(2,2) = C(m,2);

        n = 1;

        for i = 1:6
           if i==k || i==m
           else
                Cl(n,1) = C(i,1); 
                Cl(n,2) = C(i,2);
                n = n+1;
           end
        end

    else
        k = k + 1;       
        m = k + 1;        

        Cu(1,1) = C(k,1); 
        Cu(1,2) = C(k,2); 
        Cu(2,1) = C(m,1); 
        Cu(2,2) = C(m,2);

        n = 1;

        for i = 1:6
           if i==k || i==m
           else
                Cl(n,1) = C(i,1); 
                Cl(n,2) = C(i,2);
                n = n+1;
           end
        end
                      
    end

        % In the above partitioning, the equations must be grouped such that
        % Cu  be a nonsingular matrix. Hence, check its determinant:

        detCu = vpa(det(Cu),10);
        detCueval = eval(detCu)
        
end

if k == 5 & m == 6
    warndlg('Procedure failed, very likely because a real solution is not possible','!! Warning !!')
end    


% If this determinant is zero, choose other values of k  and m.
% Derive vector d, LHS of eq. (9.34a), and du, dl of eqs. (9.36a & b) 

d = maple('simplify',PuB*x45);
du(1) = d(k);
du(2) = d(m);
n = 1; 
for i = 1 : 6
    if i==k || i==m
    else
        dl(n) = d(i);
        n = n+1;
    end
end

% Then, obtain vector gamma4x45, eq. (9.37a), 
gamma4x45 = maple('simplify',Cl/Cu*du'-dl');

% Finally, derive the 4x3 matrix D1 of eq. (9.37b).  Entries of matrix D1
% are bilinear in x_4 and x_5 while y_3=[c3,s3,1]^T

for i = 1 : 4
    D1(i,1) = maple('coeff',gamma4x45(i),cos(theta(3)));
    D1(i,2) = maple('coeff',gamma4x45(i),sin(theta(3)));
    D1(i,3) = maple('simplify',gamma4x45(i)-D1(i,1)*cos(theta(3))-D1(i,2)*sin(theta(3)));
end

D1 = vpa(D1,15);

% Derive  four  3x3 submatrices of  D1 
for i = 1 : 3
    for j = 1 : 3
        subD1(i,j) = D1(i,j);
    end
end

m = 1;
n = 1;
for i = 1 : 4
    if i == 3
    else
        for j = 1 : 3
            subD2(m,n) = D1(i,j);
            n = n +1;
        end
        m = m +1;
    end
    n = 1;
end

m = 1;
n = 1;
for i = 1 : 4
    if i == 2
    else
        for j = 1 : 3
            subD3(m,n) = D1(i,j);
            n = n+1;
        end
        m = m+1;
    end
    n = 1;
end

m = 1;
n = 1;
for i = 2 : 4
    for j = 1 : 3
        subD4(m,n) = D1(i,j);
        n = n+1;
    end
    n = 1;
    m = m+1;
end

% It is clear that the trivial solution of y_3 is not admissible, since y_3 cannot be zero. 
% Hence, D1 must be rank-deficient ===> the determinants of the 3x3 submatrices 
% of D1,  subD1, subD2, subD3, and  subD4, must vanish, which leads to four equations: 
% eq1=0, eq2=0, eq3=0, eq4=0:

eq1 = maple('simplify',det(subD1));
eq2 = maple('simplify',det(subD2));
eq3 = maple('simplify',det(subD3));
eq4 = maple('simplify',det(subD4));

save eq1save eq1;
save eq2save eq2;
save eq3save eq3;
save eq4save eq4;

