%% Updated: Oct 03, 2006 for Li's manipulator (FRMS ed. 2, chap. 8)
%% Objective function of Condition Number Minimization problem.
%%
%% This function evaluates the Jacobian and returns the condition number
%% in Frobenius norm.
%%
%% The vector of joint variables must be passed to the function
%%
%% General Application: 6-axis manipulator
%% Specific Application: Fanuc ArcMate.
%%
%% Dated : Jan 01, 2004
%%
%% Notation. All matrices are capital, all scalar are small and all 
%% vector have extension _v.

function [functionvalue,H,r_v,l] = objfun(x_v)

%% functionvalue, the conditionnumber squared.
%% H, the homogenized jacobian.
%% r_v, the vector from joint origin to operation point.
%% l, the characterstic length

%% Assiging the DH parameters for a 6-axis manipulator. 
%% Notice that b(1), theta(1) and alpha(6) are taken as zero (whether or not they are zero
%% by given specs) as they do not effect the condition number.

hfigM = findobj('Tag','INPUT');
data = getappdata(hfigM, 'DATA');

a = [data.a1 data.a2 data.a3 data.a4 data.a5 data.a6];
b = [data.b1 data.b2 data.b3 data.b4 data.b5 data.b6];
alpha = [data.alpha1 data.alpha2 data.alpha3 data.alpha4 data.alpha5 data.alpha6]*pi/180;
theta = [0 x_v(2) x_v(3) x_v(4) x_v(5) x_v(6)];

setappdata(findobj('Tag','INPUT'), 'DATA', data);

overlineM=x_v(1);
M = max(max(a),max(b));
l=M/overlineM;

%% converting angles from degrees to radians
% alpha=alpha*pi/180;
theta=theta*pi/180;

% Obtaining relative rotation matrices
for i=1:6
    lambda(i)=cos(alpha(i)); mu(i)=sin(alpha(i));
    Q(:,:,i)=[cos(theta(i)) -lambda(i)*sin(theta(i)) mu(i)*sin(theta(i));
        sin(theta(i)) lambda(i)*cos(theta(i)) -mu(i)*cos(theta(i));
        0 mu(i) lambda(i)];
end

% We may confirm that Q is orthogonal.i.e. Q*Q'=Q'*Q=1
% for i=1:6
%    Q(:,:,i)*Q(:,:,i)'
%    Q(:,:,i)'*Q(:,:,i)
% end

%% Obtaining absolute rotation matrices (w.r.t. Frame-1)
P(:,:,1)=Q(:,:,1);
for i=2:6
    P(:,:,i)=P(:,:,i-1)*Q(:,:,i);
end

%% Obtaining position vector of O_i+1 w.r.t. O_i expressed in frame-i
for i=1:6
    a_v(:,i)=[a(i)*cos(theta(i)); a(i)*sin(theta(i)); b(i)];
end

%% Obtaining position vector of O_n w.r.t. O_i in frame-i and exr in frame-i
r_v(:,6)=a_v(:,6);
exr_v_i(:,6)=[-r_v(2,6);r_v(1,6);0];
for i=5:-1:1
    r_v(:,i)=a_v(:,i)+Q(:,:,i)*r_v(:,i+1);
    exr_v_i(:,i)=[-r_v(2,i);r_v(1,i);0];
end

%% Expressing "exr" in frame-1
exr_v(:,1)=exr_v_i(:,1);
for i=2:6
    exr_v(:,i)=P(:,:,i-1)*exr_v_i(:,i);
end

%% Obtaining unit vectors e_i parallel to corresponding revolute axis_i (in frame-1)

e_v(:,1)=[0;0;1];
for i=2:6
    e_v(:,i)=P(:,3,i-1);
end

%% Constructing the Jacobian

H=[e_v;-exr_v/M*overlineM];

%% F-norm condition number (Select the type of cond. # to minimize)
%KapaSquare=1/36*(trace(H*H')*trace(inv(H*H')));
 KapaSquare=1/36*(cond(H,'fro')^2);

%% 2-norm condition number
%KapaSquare=cond(H);

functionvalue=KapaSquare;

return









